From e512f66dbe348f7e47806a4ec1e493ba9c0537e5 Mon Sep 17 00:00:00 2001
From: Bharath Rupireddy <bharath.rupireddyforpostgres@gmail.com>
Date: Thu, 14 Mar 2024 07:22:08 +0000
Subject: [PATCH v14 4/4] Add invariants for shared LogwrtResult members

---
 src/backend/access/transam/xlog.c | 26 ++++++++++++++++++++++++++
 1 file changed, 26 insertions(+)

diff --git a/src/backend/access/transam/xlog.c b/src/backend/access/transam/xlog.c
index 3fd71f6787..ff65f7e07d 100644
--- a/src/backend/access/transam/xlog.c
+++ b/src/backend/access/transam/xlog.c
@@ -2581,6 +2581,32 @@ XLogWrite(XLogwrtRqst WriteRqst, TimeLineID tli, bool flexible)
 										LogwrtResult.Write);
 		pg_atomic_monotonic_advance_u64(&XLogCtl->LogwrtResult.Flush,
 										LogwrtResult.Flush);
+
+		/* Check some invariants here */
+#ifdef USE_ASSERT_CHECKING
+		{
+			XLogRecPtr	Copy = pg_atomic_read_membarrier_u64(&XLogCtl->LogwrtResult.Copy);
+			XLogRecPtr	Write = pg_atomic_read_membarrier_u64(&XLogCtl->LogwrtResult.Write);
+			XLogRecPtr	Flush = pg_atomic_read_membarrier_u64(&XLogCtl->LogwrtResult.Flush);
+
+			/*
+			 * WaitXLogInsertionsToFinish must have been called before we
+			 * reach here, so Copy ptr cannot be invalid.
+			 */
+			Assert(!XLogRecPtrIsInvalid(Copy));
+
+			/*
+			 * WAL copied to WAL buffers is always ahead of WAL written to
+			 * disk.
+			 */
+			Assert(Copy >= Write);
+
+			/*
+			 * WAL flushed to disk is always ahead of WAL written to disk.
+			 */
+			Assert(Write >= Flush);
+		}
+#endif
 	}
 }
 
-- 
2.34.1

